@@ -300,7 +300,7 @@ print_descr_default <- function(x, n = 14, perc = TRUE, digits = 2, t.table = TR
         t <- unclass(xi[[4L]])
         if(length(t) <= n) {
           if(perc) print.default(cb(t, round(t/bsum(t)*100, digits)), right = TRUE, print.gap = 2, quote = FALSE) else
-            print.table(ct(t))
+            print.table(ct(t), digits = digits)
         } else {
           t1 <- t[seq_len(n)]
           st <- bsum(t)
@@ -310,7 +310,7 @@ print_descr_default <- function(x, n = 14, perc = TRUE, digits = 2, t.table = TR
             print.default(cb(c(t1, rem), round(c(pct, 100-bsum(pct)), digits)), right = TRUE, print.gap = 2, quote = FALSE)
             # cat("...\n")
           } else {
-            print.table(ct(c(t1, rem)))
+            print.table(ct(c(t1, rem)), digits = digits)
             # cat("...\n")
           }
           if(summary) {
@@ -371,7 +371,14 @@ print_descr_grouped <- function(x, n = 14, perc = TRUE, digits = 2, t.table = TR
                                  ncol = 2, dimnames =  list(NULL, c("Min", "Max")))),
                     quote = FALSE, right = TRUE, print.gap = 2)
     } else {
-      if(perc) stat <- cbind(stat[, 1L, drop = FALSE], Perc = stat[, 1L]/bsum(stat[, 1L])*100, stat[, -1L, drop = FALSE])
+      if(perc) {
+        if(wsuml && ncol(stat) > 4L) { # If weights and non-character
+          ncolf <- 1:(2L + (dimnames(stat)[[2L]][2L] == "Ndist"))
+          stat <- if(wsuml) cbind(stat[, ncolf, drop = FALSE], Perc = stat[, "WeightSum"]/bsum(stat[, "WeightSum"])*100, stat[, -ncolf, drop = FALSE])
+        } else {
+          stat <- cbind(stat[, 1L, drop = FALSE], Perc = stat[, 1L]/bsum(stat[, 1L])*100, stat[, -1L, drop = FALSE])
+        }
+      }
       print.qsu(stat, digits)
     }
     if(length(xi) > 3L) { # Table or quantiles
@@ -391,7 +398,7 @@ print_descr_grouped <- function(x, n = 14, perc = TRUE, digits = 2, t.table = TR
           if(perc) pct <- tab %r/% st * 100 # dimnames(tab)[[2L]] <- paste0(dimnames(tab)[[2L]], "\nFreq  Perc")
         }
         if(perc) {
-          tab <- duplAttributes(paste(tab, format(pct, digits = digits, justify = "right")), tab)
+          tab <- duplAttributes(paste(format(tab, digits = digits, justify = "right"), format(pct, digits = digits, justify = "right")), tab)
           print.default(if(t.table) tab else t(tab), right = TRUE, print.gap = 2, quote = FALSE)
         } else print.table(if(t.table) tab else t(tab), digits = digits)
         if(summary && nrow(t) > n) {
